/*
 * Copyright (C) 2023 ~ 2024 UnionTech Software Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "offlinecheck.h"
#include "common.h"

#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QProcess>
#include <QJsonDocument>
#include <QFile>
#include <QJsonObject>
#include <QJsonArray>
#include <QDebug>

const int redundant_space = 10 * 1024;

OfflineCheck::OfflineCheck(QObject *parent)
    : QObject(parent)
{
}

OfflineCheck::~OfflineCheck()
{
}

// 开始进行离线包检测
void OfflineCheck::startOfflineFileCheck(const QString &homepath, const QString &filepath)
{
    Q_EMIT sendOfflineUpgradeLog("离线oup包检测开始。。。");
    // 检测-离线包解压空间 -需要2倍文件大小的空闲空间
    if (!checkOfflineFile(filepath)) {
        Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::OfflineFileArXSpace, false);
        Q_EMIT sendOfflineUpgradeLog("oup文件解压需要空间不足，请清理");
        return;
    }

    QFile file(filepath);
    if (file.exists()) {
        QString cmd = QString("/usr/bin/file \"%1\"").arg(filepath);
        QProcess proc;
        proc.start(cmd);
        proc.waitForStarted();
        proc.waitForFinished();
        QString sfiletype = QString::fromLocal8Bit(proc.readAllStandardOutput()).trimmed();
        qInfo() << Q_FUNC_INFO << QString("oup file type query: %1").arg(sfiletype);
        proc.close();

        if (sfiletype.contains("Squashfs")) {
            m_isSquashfs = true;
        } else {
            m_isSquashfs = false;
        }

    } else {
        Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::OfflineFileDirComplete, false);
        return;
    }

    if (m_isSquashfs) {
        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -检测文件完整性");
        // 检测-离线包目录完整性
        if (!checkOfflineFileDirComplete(homepath, filepath)) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::OfflineFileDirComplete, false);
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -检测失败，请重新制作oup文件");
            return;
        }

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -hooks前置脚本检测");
        // 安装前置检测
        if (!checkHookPre(homepath)) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::HooksPre, false);
            return;
        }

        // 检测-升级类型（系统升级或者CVE漏洞升级）
        checkOfflineUpgradeType(homepath);

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -系统版本检测");
        // 检测-系统版本
        if (!checkSystemVersion()) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::SystemVersion, false);
            return;
        }

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -oup架构检测");
        // 检测-离线包架构
        if (!checkOfflineFileArch()) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::OfflineFileArch, false);
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -请检测oup文件是否与系统架构一致");
            return;
        }
        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -系统备份空间检测");
        // 检测-系统备份空间
        if (!checkSystemBackupSpace()) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::SystemBackupSpace, false);
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -空间不足，务必保证系统盘已占空间+10G < 数据盘空闲空间");
            return;
        }

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -可更新数量检测");
        // 检测-更新包数量
        if (!checkUpgradeSum(homepath)) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::UpgradeSum, false);
            return;
        } else {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::UpgradeSum, true);
            return;
        }
    } else {
        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -检测文件完整性");
        // 检测-离线包目录完整性
        if (!checkArOfflineFileDirComplete(homepath, filepath)) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::OfflineFileDirComplete, false);
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -检测失败，请重新制作oup文件");
            return;
        }

        // 检测-升级类型（系统升级或者CVE漏洞升级）
        checkArOfflineUpgradeType(homepath);

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -系统版本检测");
        // 检测-系统版本
        if (!checkArSystemVersion()) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::SystemVersion, false);
            return;
        }

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -oup架构检测");
        // 检测-离线包架构
        if (!checkArOfflineFileArch()) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::OfflineFileArch, false);
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -请检测oup文件是否与系统架构一致");
            return;
        }
        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -系统备份空间检测");
        // 检测-系统备份空间
        if (!checkArSystemBackupSpace()) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::SystemBackupSpace, false);
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -空间不足，务必保证系统盘已占空间+10G < 数据盘空闲空间");
            return;
        }

        Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -可更新数量检测");
        // 检测-更新包数量
        if (!checkArUpgradeSum(homepath)) {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::UpgradeSum, false);
            return;
        } else {
            Q_EMIT sendOfflineFileCheckResult(OfflineCheckItem::UpgradeSum, true);
            return;
        }
    }
}

bool OfflineCheck::checkOfflineFile(const QString &filepath)
{
    QFile file(filepath);
    if (file.exists()) {
        // 查询文件大小
        qint64 size = file.size();
        // 查询空闲空间
        QString freeSize = getHomeFreeSize();

        // 检测离线文件需要2倍空闲大小
        qInfo() << Q_FUNC_INFO << QString("redundant space size: %1Mb").arg(redundant_space);
        qInfo() << Q_FUNC_INFO << QString("Offline file size: %1Mb").arg(size / 1024 / 1024);
        qInfo() << Q_FUNC_INFO << QString("System free space size: %1Mb").arg(freeSize.toInt() / 1024);
        if (size / 1024 / 1024 * 2 + redundant_space >= freeSize.toInt() / 1024) {
            return false;
        }
    } else {
        qWarning() << Q_FUNC_INFO << QString("%1 not exist").arg(filepath);
        return false;
    }
    return true;
}

bool OfflineCheck::checkOfflineFileDirComplete(const QString &homepath, const QString &filepath)
{
    QString targetPath = homepath + OFFLINE_COPY_SQUASHFS_PATH;
    QString oupPath = targetPath + OFFLINE_COPY_OUP_PATH;
    QFile file(filepath);
    if (file.exists()) {
        QDir dir;
        if (dir.exists(targetPath)) {
            // 如果deepin-offline-update目录存在，则删除
            // 取消repo.sfs挂载
            QProcess repoProc;
            repoProc.setWorkingDirectory(targetPath);
            repoProc.setProcessChannelMode(QProcess::MergedChannels);
            repoProc.start("umount", QStringList() << targetPath + OFFLINE_DIR_REPO);
            repoProc.waitForFinished();
            repoProc.close();
            // 取消hooks.sfs挂载
            QProcess hooksProc;
            hooksProc.setWorkingDirectory(targetPath);
            hooksProc.setProcessChannelMode(QProcess::MergedChannels);
            hooksProc.start("umount", QStringList() << targetPath + OFFLINE_DIR_HOOKS);
            hooksProc.waitForFinished();
            hooksProc.close();
            // 取消deepin-offline-update/oup挂载
            QProcess offlineProc;
            offlineProc.setWorkingDirectory(targetPath);
            offlineProc.setProcessChannelMode(QProcess::MergedChannels);
            offlineProc.start("umount", QStringList() << oupPath);
            offlineProc.waitForFinished();
            offlineProc.close();

            // 删除deepin-offline-update目录所有文件
            dir.setPath(targetPath);
            dir.removeRecursively();
        }
        if (!dir.mkpath(targetPath)) {
            qWarning() << Q_FUNC_INFO << QString("Failed mkdir dir %1:").arg(targetPath);
            return false;
        }

        if (!dir.mkpath(oupPath)) {
            qWarning() << Q_FUNC_INFO << QString("Failed mkdir dir %1:").arg(oupPath);
            return false;
        }
        // 挂载.oup文件到/deepin-offline-update/oup目录
        QProcess builder;
        builder.setWorkingDirectory(targetPath);
        builder.setProcessChannelMode(QProcess::MergedChannels);
        builder.start("mount", QStringList() << filepath << oupPath);
        if (!builder.waitForFinished(-1)) {
            qWarning() << Q_FUNC_INFO << QString("Failed mount oup path:") << builder.errorString();
            builder.close();
            return false;
        } else {
            builder.close();

            // 判断离线包解压文件是否完整
            QFile fileInfo(oupPath + OFFLINE_ARX_INFO);
            QFile fileBinary(oupPath + OFFLINE_ARX_BINARY);
            QFile fileRepo(oupPath + OFFLINE_ARX_REPO);
            if (!fileInfo.exists() || !fileBinary.exists() || !fileRepo.exists()) {
                return false;
            }
        }
    } else {
        qWarning() << Q_FUNC_INFO << QString("%1 not exist").arg(filepath);
        return false;
    }

    return true;
}

void OfflineCheck::checkOfflineUpgradeType(const QString &homepath)
{
    QString targetPath = homepath + OFFLINE_COPY_SQUASHFS_PATH;
    QString oupPath = targetPath + OFFLINE_COPY_OUP_PATH;

    // json解析
    QJsonDocument appInfoDoc;
    QJsonParseError err;
    // 读app.json获取名称和路径信息
    QFile appfile(oupPath + "info.json");
    if (!appfile.open(QFile::ReadOnly)) {
        qInfo() << "can not find app info file";
    }

    QJsonObject object = QJsonDocument::fromJson(appfile.readAll(), &err).object();
    if (QJsonParseError::NoError != err.error) {
        qInfo() << "incorrect json format";
    }
    appfile.close();

    // 升级类型
    QJsonValue typeValue = object.value("type");
    int type = typeValue.toInt();

    // 漏洞信息
    QJsonValue dataValue = object.value("data");
    QJsonObject dataObject = dataValue.toObject();
    QJsonValue valueSystemType = dataObject.value("systemType");
    QJsonValue valueID = dataObject.value("cveId");
    QJsonValue valueArchs = dataObject.value("archs");
    m_offlineType = valueSystemType.toString();
    QString cveid = valueID.toString();
    m_offlineArchs = valueArchs.toString();

    if (UpgradeType::CVEPatchUpgrade == type) {
        m_upgradeType = UpgradeType::CVEPatchUpgrade;
        m_patchUpgradeSerial = cveid;
    } else {
        m_upgradeType = UpgradeType::SystemUpgrade;
    }

    qWarning() << Q_FUNC_INFO << QString("offline arch: %1").arg(m_offlineArchs);
    qWarning() << Q_FUNC_INFO << QString("offline m_offlineType: %1").arg(m_offlineType);
    qWarning() << Q_FUNC_INFO << QString("offline type: %1").arg(type);
}

bool OfflineCheck::checkSystemVersion()
{
    // 系统版本 -专业版/服务器版
    QString offlineVersion = m_offlineType;

    //    if (offlineVersion != SystemEditionName) {
    //        qWarning() << Q_FUNC_INFO << QString("offline version:%1 system version:%2").arg(offlineVersion).arg(SystemEditionName);
    //        return false;
    //    }

    return true;
}

bool OfflineCheck::checkOfflineFileArch()
{
    QString offlineFileArch = m_offlineArchs.toLower();

    QProcess procArch;
    procArch.start("uname -m");
    procArch.waitForStarted();
    procArch.waitForFinished();
    QString sProcessArchName = QString::fromLocal8Bit(procArch.readAllStandardOutput()).trimmed();
    procArch.close();

    if (sProcessArchName == ARCH_AMD || sProcessArchName == CRP_ARCH_I386) {
        if (offlineFileArch == CRP_ARCH_AMD) {
            return true;
        }
    }
    if (sProcessArchName == ARCH_ARM && offlineFileArch == CRP_ARCH_ARM) {
        return true;
    }
    if (sProcessArchName == ARCH_MIPS && offlineFileArch == CRP_ARCH_MIPS) {
        return true;
    }
    if (sProcessArchName == ARCH_SW && offlineFileArch == CRP_ARCH_SW) {
        return true;
    }
    if (sProcessArchName == ARCH_LOONGARCH && offlineFileArch == CRP_ARCH_LOONGARCH) {
        return true;
    }

    qWarning() << Q_FUNC_INFO << QString("offline file arch check failed: %1") << offlineFileArch;
    return false;
}

bool OfflineCheck::checkSystemBackupSpace()
{
    // 查询系统空闲空间
    QString freeSize = getHomeFreeSize();

    // 查询根目录已用空间
    QString cmd = QString("/usr/bin/df %1").arg(SYSTEM_DIR_PATH);
    QProcess procFree;
    procFree.start(cmd);
    procFree.waitForStarted();
    procFree.waitForFinished();
    QString sProcessResultPath = QString::fromLocal8Bit(procFree.readAllStandardOutput()).trimmed();
    procFree.close();
    QString memerysize = sProcessResultPath.split("\n").at(1);
    QString UsedSize = memerysize.split(" ", QString::SkipEmptyParts).at(2);

    qInfo() << Q_FUNC_INFO << QString("redundant space size: %1Kb").arg(redundant_space * 1024);
    qInfo() << Q_FUNC_INFO << QString("root dir used size: %1Kb").arg(UsedSize.toInt());
    qInfo() << Q_FUNC_INFO << QString("data dir free size: %1Kb").arg(freeSize);
    if (freeSize.toInt() < UsedSize.toInt() + redundant_space * 1024) {
        qWarning() << Q_FUNC_INFO << QString("Insufficient free space, free size:%1 used size:%2").arg(freeSize).arg(UsedSize);
        return false;
    }

    return true;
}

bool OfflineCheck::checkUpgradeSum(const QString &homepath)
{
    QString targetPath = homepath + OFFLINE_COPY_SQUASHFS_PATH;
    QString oupPath = targetPath + OFFLINE_COPY_OUP_PATH;

    QString repoPath = targetPath + OFFLINE_DIR_REPO;
    QDir repoDir;
    //创建之前，先检查repo目录是否存在
    if (!repoDir.exists(repoPath)) {
        repoDir.mkdir(repoPath);
    }

    // mount repo.sfs文件系统到repo目录
    QProcess builder;
    builder.setWorkingDirectory(targetPath);
    builder.setProcessChannelMode(QProcess::MergedChannels);
    builder.start("mount", QStringList() << oupPath + OFFLINE_ARX_REPO << targetPath + OFFLINE_DIR_REPO);
    if (!builder.waitForFinished()) {
        qWarning() << Q_FUNC_INFO << QString("Failed mount %1").arg(oupPath + OFFLINE_ARX_REPO) << builder.errorString();
        builder.close();
        return false;
    }
    builder.close();

    // 删除lists目录
    QString aptlistPath = "/var/lib/upgrade/lists/";
    QDir dir;
    if (dir.exists(aptlistPath)) {
        dir.setPath(aptlistPath);
        dir.removeRecursively();
    }
    // 根据仓库配置自动识别源地址
    QDir poolDir(repoPath + "/pool");
    QString poolName;
    for (const QFileInfo &i : poolDir.entryInfoList(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Hidden | QDir::NoSymLinks)) {
        QString absolutePath = i.absoluteFilePath();
        poolName = poolName + QString(" %1").arg(absolutePath.split("/", QString::SkipEmptyParts).last());
    }

    // 修改update.list文件
    QString updateInfo = QString("deb [trusted=yes] file://%1/.cache/deepin-offline-update/repo/ eagle%2").arg(homepath).arg(poolName);
    QFile file(SYSTEM_SOURCE_UPDATE_LIST);
    if (file.open(QIODevice::ReadWrite)) {
        file.resize(0);
        QTextStream stream(&file);
        file.seek(0);
        stream << updateInfo;
        file.close();
    }

    // 指定目录仓库
    QString cmd = QString("apt -o Dir::Etc::sourcelist=%1 -o Dir::State::lists=/var/lib/upgrade/lists/ -o Dir::Etc::sourceparts=/dev/null update").arg(SYSTEM_SOURCE_UPDATE_LIST);
    QProcess processSource;
    processSource.setProcessChannelMode(QProcess::MergedChannels);
    processSource.start(cmd);
    if (!processSource.waitForFinished()) {
        qWarning() << Q_FUNC_INFO << QString("Failed to connect sourcelist %1").arg(SYSTEM_SOURCE_UPDATE_LIST) << processSource.errorString();
        processSource.close();
        return false;
    }
    QString updatesource = QString::fromLocal8Bit(processSource.readAllStandardOutput()).trimmed();
    processSource.close();

    // apt update 执行结果
    Q_EMIT sendOfflineUpgradeLog("apt update执行结果");
    Q_EMIT sendOfflineUpgradeLog(updatesource);
    // 查询可更新数量
    m_distUpgradeSum = getUpgradeListSum();
    // 可更新数量为0,则不允许更新
    if (m_distUpgradeSum == 0) {
        return false;
    }
    return true;
}

bool OfflineCheck::checkArOfflineFileDirComplete(const QString &homepath, const QString &filepath)
{
    QString targetPath = homepath + OFFLINE_COPY_AR_PATH;
    QFile file(filepath);
    if (file.exists()) {
        QDir dir;
        if (dir.exists(targetPath)) {
            dir.setPath(targetPath);
            dir.removeRecursively();
        }
        if (!dir.mkpath(targetPath)) {
            qWarning() << Q_FUNC_INFO << QString("Failed mkdir dir %1:").arg(targetPath);
            return false;
        }

        QFileInfo info(filepath);
        if (file.copy(filepath, targetPath + info.fileName())) {
            // 解压 .oup文件
            QProcess builder;
            builder.setWorkingDirectory(targetPath);
            builder.setProcessChannelMode(QProcess::MergedChannels);
            builder.start("ar", QStringList() << "-x" << targetPath + info.fileName());
            if (!builder.waitForFinished()) {
                qWarning() << Q_FUNC_INFO << QString("Failed ar -x %1:").arg(targetPath + info.fileName()) << builder.errorString();
                builder.close();
                return false;
            } else {
                builder.close();
                // 删除.oup文件
                QFile offlineFile(targetPath + info.fileName());
                if (offlineFile.exists()) {
                    offlineFile.remove();
                }

                // 判断离线包解压文件是否完整
                QFile fileInfo(targetPath + OFFLINE_ARX_INFO);
                QFile fileBinary(targetPath + OFFLINE_ARX_BINARY);
                QFile fileRepo(targetPath + OFFLINE_ARX_REPO);
                if (!fileInfo.exists() || !fileBinary.exists() || !fileRepo.exists()) {
                    return false;
                }
            }
        } else {
            qWarning() << Q_FUNC_INFO << QString("Failed copy %1 to %2").arg(filepath).arg(targetPath + info.fileName());
            return false;
        }
    } else {
        qWarning() << Q_FUNC_INFO << QString("%1 not exist").arg(filepath);
        return false;
    }

    return true;
}

void OfflineCheck::checkArOfflineUpgradeType(const QString &homepath)
{
    QString targetPath = homepath + OFFLINE_COPY_AR_PATH;
    // json解析
    QJsonDocument appInfoDoc;
    QJsonParseError err;
    // 读app.json获取名称和路径信息
    QFile appfile(targetPath + "info.json");
    if (!appfile.open(QFile::ReadOnly)) {
        qInfo() << "can not find app info file";
    }

    QJsonObject object = QJsonDocument::fromJson(appfile.readAll(), &err).object();
    if (QJsonParseError::NoError != err.error) {
        qInfo() << "incorrect json format";
    }
    appfile.close();

    // 升级类型
    QJsonValue typeValue = object.value("type");
    int type = typeValue.toInt();

    // 漏洞信息
    QJsonValue dataValue = object.value("data");
    QJsonObject dataObject = dataValue.toObject();
    QJsonValue valueSystemType = dataObject.value("systemType");
    QJsonValue valueID = dataObject.value("cveId");
    QJsonValue valueArchs = dataObject.value("archs");
    m_offlineType = valueSystemType.toString();
    QString cveid = valueID.toString();
    m_offlineArchs = valueArchs.toString();

    if (UpgradeType::CVEPatchUpgrade == type) {
        m_upgradeType = UpgradeType::CVEPatchUpgrade;
        m_patchUpgradeSerial = cveid;
    } else {
        m_upgradeType = UpgradeType::SystemUpgrade;
    }
}

bool OfflineCheck::checkArSystemVersion()
{
    // 系统版本 -专业版/服务器版
    QString offlineVersion = m_offlineType;

    //    if (offlineVersion != SystemEditionName) {
    //        qWarning() << Q_FUNC_INFO << QString("offline version:%1 system version:%2").arg(offlineVersion).arg(SystemEditionName);
    //        return false;
    //    }

    return true;
}

bool OfflineCheck::checkArOfflineFileArch()
{
    QString offlineFileArch = m_offlineArchs.toLower();

    QProcess procArch;
    procArch.start("uname -m");
    procArch.waitForStarted();
    procArch.waitForFinished();
    QString sProcessArchName = QString::fromLocal8Bit(procArch.readAllStandardOutput()).trimmed();
    procArch.close();

    if (sProcessArchName == ARCH_AMD || sProcessArchName == CRP_ARCH_I386) {
        if (offlineFileArch == CRP_ARCH_AMD) {
            return true;
        }
    }
    if (sProcessArchName == ARCH_ARM && offlineFileArch == CRP_ARCH_ARM) {
        return true;
    }
    if (sProcessArchName == ARCH_MIPS && offlineFileArch == CRP_ARCH_MIPS) {
        return true;
    }
    if (sProcessArchName == ARCH_SW && offlineFileArch == CRP_ARCH_SW) {
        return true;
    }
    if (sProcessArchName == ARCH_LOONGARCH && offlineFileArch == CRP_ARCH_LOONGARCH) {
        return true;
    }

    return false;
}

bool OfflineCheck::checkArSystemBackupSpace()
{
    // 查询系统空闲空间
    QString freeSize = getHomeFreeSize();

    // 查询根目录已用空间
    QString cmd = QString("/usr/bin/df %1").arg(SYSTEM_DIR_PATH);
    QProcess procFree;
    procFree.start(cmd);
    procFree.waitForStarted();
    procFree.waitForFinished();
    QString sProcessResultPath = QString::fromLocal8Bit(procFree.readAllStandardOutput()).trimmed();
    procFree.close();
    QString memerysize = sProcessResultPath.split("\n").at(1);
    QString UsedSize = memerysize.split(" ", QString::SkipEmptyParts).at(2);

    qInfo() << Q_FUNC_INFO << QString("redundant space size: %1Kb").arg(redundant_space * 1024);
    qInfo() << Q_FUNC_INFO << QString("root dir used size: %1Kb").arg(UsedSize.toInt());
    qInfo() << Q_FUNC_INFO << QString("data dir free size: %1Kb").arg(freeSize);
    if (freeSize.toInt() < UsedSize.toInt() + redundant_space * 1024) {
        qWarning() << Q_FUNC_INFO << QString("Insufficient free space, free size:%1 used size:%2").arg(freeSize).arg(UsedSize);
        return false;
    }

    return true;
}

bool OfflineCheck::checkArUpgradeSum(const QString &homepath)
{
    QString targetPath = homepath + OFFLINE_COPY_AR_PATH;

    QString repoPath = targetPath + OFFLINE_DIR_REPO;
    QDir repoDir;
    //创建之前，先检查该目录是否存在
    if (!repoDir.exists(repoPath)) {
        repoDir.mkdir(repoPath);
    }

    // 修改update.list文件
    QString updateInfo = QString("deb [trusted=yes] file://%1/.cache/deepin-offline-update-ar/repo/ eagle main").arg(homepath);
    QFile file(SYSTEM_SOURCE_UPDATE_LIST);
    if (file.open(QIODevice::ReadWrite)) {
        file.resize(0);
        QTextStream stream(&file);
        file.seek(0);
        stream << updateInfo;
        file.close();
    }

    // 解压 repo.tar文件
    QProcess builder;
    builder.setWorkingDirectory(targetPath);
    builder.setProcessChannelMode(QProcess::MergedChannels);
    builder.start("mount", QStringList() << targetPath + OFFLINE_ARX_REPO << targetPath + OFFLINE_DIR_REPO);
    if (!builder.waitForFinished()) {
        qWarning() << Q_FUNC_INFO << QString("Failed mount %1").arg(targetPath + OFFLINE_ARX_REPO) << builder.errorString();
        builder.close();
        return false;
    }
    builder.close();

    // 删除lists目录
    QString aptlistPath = "/var/lib/upgrade/lists/";
    QDir dir;
    if (dir.exists(aptlistPath)) {
        dir.setPath(aptlistPath);
        dir.removeRecursively();
    }

    // 指定目录仓库
    QString cmd = QString("apt -o Dir::Etc::sourcelist=%1 -o Dir::State::lists=/var/lib/upgrade/lists/ -o Dir::Etc::sourceparts=/dev/null update").arg(SYSTEM_SOURCE_UPDATE_LIST);
    QProcess processSource;
    processSource.setProcessChannelMode(QProcess::MergedChannels);
    processSource.start(cmd);
    if (!processSource.waitForFinished()) {
        qWarning() << Q_FUNC_INFO << QString("Failed to connect sourcelist %1").arg(SYSTEM_SOURCE_UPDATE_LIST) << processSource.errorString();
        processSource.close();
        return false;
    }
    QString updatesource = QString::fromLocal8Bit(processSource.readAllStandardOutput()).trimmed();
    processSource.close();

    // apt update 执行结果
    Q_EMIT sendOfflineUpgradeLog("apt update执行结果");
    Q_EMIT sendOfflineUpgradeLog(updatesource);
    // 查询可更新数量
    m_distUpgradeSum = getUpgradeListSum();
    // 可更新数量为0,则不允许更新
    if (m_distUpgradeSum == 0) {
        return false;
    }
    return true;
}

bool OfflineCheck::checkHookPre(const QString &homepath)
{
    QString targetPath = homepath + OFFLINE_COPY_SQUASHFS_PATH;
    QString oupPath = targetPath + OFFLINE_COPY_OUP_PATH;

    QString hooksPath = targetPath + OFFLINE_DIR_HOOKS;
    QDir hooksDir;
    //创建之前，先检查hooks目录是否存在
    if (!hooksDir.exists(hooksPath)) {
        hooksDir.mkdir(hooksPath);
    }

    // mount hooks.sfs文件系统到hooks目录
    QProcess builder;
    builder.setWorkingDirectory(targetPath);
    builder.setProcessChannelMode(QProcess::MergedChannels);
    builder.start("mount", QStringList() << oupPath + OFFLINE_ARX_HOOKS << targetPath + OFFLINE_DIR_HOOKS);
    if (!builder.waitForFinished()) {
        qWarning() << Q_FUNC_INFO << QString("Failed mount %1").arg(oupPath + OFFLINE_ARX_HOOKS) << builder.errorString();
        builder.close();
        return false;
    }
    builder.close();

    // 遍历hooks/prehook目录
    QString prehook = hooksPath + "/prehook";
    QDir dir(prehook);
    for (const QFileInfo &i : dir.entryInfoList(QDir::NoDotAndDotDot | QDir::Files | QDir::Dirs | QDir::Hidden | QDir::NoSymLinks)) {
        QString filepath = i.absoluteFilePath();

        QProcess builder;
        builder.setWorkingDirectory(prehook);
        builder.setProcessChannelMode(QProcess::MergedChannels);
        builder.start(QString("/bin/bash %1 %2").arg(filepath).arg(homepath));
        if (!builder.waitForFinished()) {
            qWarning() << Q_FUNC_INFO << QString("Failed bash %1").arg(prehook) << builder.errorString();
            builder.close();
            return false;
        }

        // 安装前置检测信息输出
        QString info = QString::fromLocal8Bit(builder.readAllStandardOutput()).trimmed();
        // 退出，0检测成功，1检测失败
        if (builder.exitCode()) {
            Q_EMIT sendOfflineUpgradeLog("离线oup包检测 -hooks前置脚本执行失败，失败详情请:");
            Q_EMIT sendOfflineUpgradeLog(info);
            return false;
        }
        builder.close();
    }

    return true;
}

// 执行安装成功后hook脚本
void OfflineCheck::execHookPost(const QString &homepath)
{
    QString targetPath = homepath + OFFLINE_COPY_SQUASHFS_PATH;
    QString hooksPath = targetPath + OFFLINE_DIR_HOOKS;

    // 遍历hooks/posthook目录
    QString prehook = hooksPath + "/posthook";
    QDir dir(prehook);
    for (const QFileInfo &i : dir.entryInfoList(QDir::NoDotAndDotDot | QDir::Files | QDir::Dirs | QDir::Hidden | QDir::NoSymLinks)) {
        QString filepath = i.absoluteFilePath();

        QProcess builder;
        builder.setWorkingDirectory(prehook);
        builder.setProcessChannelMode(QProcess::MergedChannels);
        builder.start(QString("/bin/bash \"%1\" %2").arg(filepath).arg(homepath));
        if (!builder.waitForFinished()) {
            qWarning() << Q_FUNC_INFO << QString("Failed bash %1").arg(prehook) << builder.errorString();
            builder.close();
        }

        builder.close();
    }
}

QString OfflineCheck::getHomeFreeSize()
{
    // 查询.cache空闲大小
    QString cmd = QString("/usr/bin/df %1").arg(CACHE_DIR_PATH);
    QProcess procFree;
    procFree.start(cmd);
    procFree.waitForStarted();
    procFree.waitForFinished();
    QString sProcessResultPath = QString::fromLocal8Bit(procFree.readAllStandardOutput()).trimmed();
    procFree.close();
    QString memerysize = sProcessResultPath.split("\n").at(1);
    QString freeSize = memerysize.split(" ", QString::SkipEmptyParts).at(3);

    return freeSize;
}

int OfflineCheck::execSystemUpgrade()
{
    Q_EMIT sendOfflineUpgradeLog("apt dist upgrade结果");
    QStringList arguments;
    // 直接将参数传递给 QProcess，不通过 bash
    arguments << "-y"
              << "-o"
              << "Dpkg::Options::=--force-confnew"
              << "-o"
              << "Dir::Etc::sourcelist=/etc/deepin-offline-update/update.list"
              << "-o"
              << "Dir::State::lists=/var/lib/upgrade/lists/"
              << "-o"
              << "Dir::Etc::sourceparts=/dev/null"
              << "dist-upgrade";

    // 设置环境变量
    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
    env.insert("DEBIAN_FRONTEND", "noninteractive");
    env.insert("DEBCONF_NONINTERACTIVE_SEEN", "true");
    m_upgradeProcess.setProcessEnvironment(env);

    // 设置要执行的程序（apt）
    m_upgradeProcess.setProgram("/usr/bin/apt");
    m_upgradeProcess.setArguments(arguments);

    // 连接输出信号
    connect(&m_upgradeProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(onReadData()));
    m_upgradeProcess.setReadChannel(QProcess::StandardOutput);

    // 启动进程
    m_upgradeProcess.start();
    m_upgradeProcess.waitForFinished(-1);

    int exitcode = m_upgradeProcess.exitCode();
    Q_EMIT sendOfflineUpgradeLog(m_upgradeProcess.readAllStandardError());

    return exitcode;
}

void OfflineCheck::onReadData()
{
    QString info = m_upgradeProcess.readAllStandardOutput();
    Q_EMIT sendExecDistUpgradeInfo(info);
}

int OfflineCheck::getUpgradeListSum()
{
    int sum = 0;
    QString cmdSum = QString("apt -o Dir::Etc::sourcelist=%1 -o Dir::State::lists=/var/lib/upgrade/lists/ -o Dir::Etc::sourceparts=/dev/null list --upgradable").arg(SYSTEM_SOURCE_UPDATE_LIST);
    QProcess processSum;
    processSum.start(cmdSum);
    processSum.waitForStarted();
    processSum.waitForFinished();
    QString info = QString::fromLocal8Bit(processSum.readAllStandardOutput()).trimmed();
    processSum.close();
    QStringList distUpgradeList = info.split("\n");
    for (int i = 0; i < distUpgradeList.count(); ++i) {
        if (distUpgradeList.at(i).contains("[")) {
            sum += 1;
        }
    }

    Q_EMIT sendOfflineUpgradeLog("apt lists --upgradable结果");
    Q_EMIT sendOfflineUpgradeLog(info);
    return sum;
}
