/*
 * Copyright (C) 2023 ~ 2024 UnionTech Software Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "writedbusdata.h"
#include "common.h"
#include "invokers/invokerfactory.h"
#include "offlinecheck/offlinecheck.h"

#include <QtConcurrent/QtConcurrent>
#include <QDBusContext>

#define BACK_PROGRESS_VALUE_A 100
#define BACK_PROGRESS_VALUE_B 110
#define BACK_PROGRESS_VALUE_C 120
#define BACK_PROGRESS_VALUE_D 130
#define BACK_PROGRESS_VALUE_E 140
#define BACK_PROGRESS_VALUE_F 199

// 合法调用进程路径列表
const QStringList ValidInvokerExePathList =
    {"/usr/bin/deepin-offline-update-tool"};

WriteDBusData::WriteDBusData(QObject *parent)
    : QObject(parent)
    , m_adaptor(new DaemonAdaptor(this))
    , m_ABRecoveryDBusInter(nullptr)
    , m_offlineCheck(nullptr)
{
    if (!QDBusConnection::systemBus().registerService(service) || !QDBusConnection::systemBus().registerObject(path, this)) {
        exit(0);
    }

    // 初始化离线检测
    m_offlineCheck = new OfflineCheck();
    connect(m_offlineCheck, &OfflineCheck::sendOfflineFileCheckResult, this, &WriteDBusData::acceptOfflineFileCheckResult);
    connect(m_offlineCheck, &OfflineCheck::sendExecDistUpgradeInfo, this, &WriteDBusData::acceptExecDistUpgradeInfo);
    connect(m_offlineCheck, &OfflineCheck::sendOfflineUpgradeLog, this, &WriteDBusData::acceptOfflineUpgradeLog);

    m_ABRecoveryDBusInter = InvokerFactory::GetInstance().CreateInvoker("com.deepin.ABRecovery",
                                                                        "/com/deepin/ABRecovery",
                                                                        "com.deepin.ABRecovery",
                                                                        ConnectType::SYSTEM, this);
    m_ABRecoveryDBusInter->Connect("JobEnd", this, SLOT(acceptBackupSystemProgress(QString, bool, QString)));
}

WriteDBusData::~WriteDBusData()
{
}

void WriteDBusData::StartApp()
{
}

void WriteDBusData::ExitApp()
{
    qApp->quit();
}

// 开始进行离线包检测
void WriteDBusData::StartOfflineFileCheck(QString homepath, QString filepath)
{
    if (!isValidInvoker()) {
        return;
    }

    QDBusConnection conn = connection();
    QDBusMessage msg = message();
    QString senderBusName = msg.service();

    Authority::Result result;
    result = Authority::instance()->checkAuthorizationSync("com.deepin.deepin-offline.checkAuthentication", SystemBusNameSubject(senderBusName),
                                                           Authority::AllowUserInteraction);

    if (result == Authority::Yes) {
        m_homepath = homepath;
        m_offlineCheck->startOfflineFileCheck(homepath, filepath);
    } else {
        acceptOfflineFileCheckResult(OfflineCheckItem::CancelUpdate, false);
        acceptOfflineUpgradeLog("用户取消检测");
    }
}
void WriteDBusData::acceptOfflineFileCheckResult(int itemType, bool result)
{
    Q_EMIT NotifyOfflineFileCheckResult(itemType, result);
}
void WriteDBusData::acceptExecDistUpgradeInfo(QString info)
{
    Q_EMIT NotifyExecDistUpgradeInfo(info);
}

void WriteDBusData::acceptOfflineUpgradeLog(QString log)
{
    Q_EMIT NotifyOfflineUpgradeLog(log);
}

int WriteDBusData::GetUpgradeType()
{
    return m_offlineCheck->getUpgradeType();
}
QString WriteDBusData::GetPatchUpgradeSerial()
{
    return m_offlineCheck->getPatchUpgradeSerial();
}
int WriteDBusData::GetDistUpgradeSum()
{
    return m_offlineCheck->getDistUpgradeSum();
}

bool WriteDBusData::IsAllowBackupSystem()
{
    bool isFlag = false;
    QDBusMessage msg = DBUS_BLOCK_INVOKE(m_ABRecoveryDBusInter, "CanBackup");
    if (!msg.errorMessage().isEmpty() || !msg.errorName().isEmpty()) {
        qWarning() << Q_FUNC_INFO << msg.errorMessage();
    } else {
        GET_MESSAGE_VALUE(bool, status, msg);
        isFlag = status;
    }

    return isFlag;
}

void WriteDBusData::BackupSystem()
{
    if (!isValidInvoker()) {
        return;
    }

    Q_EMIT NotifyExecDistUpgradeInfo("开始进行系统备份...");
    Q_EMIT NotifyOfflineUpgradeProgress(30, 1);

    if (IsAllowBackupSystem()) {
        QDBusMessage msg = DBUS_BLOCK_INVOKE(m_ABRecoveryDBusInter, "StartBackup");
        if (!msg.errorMessage().isEmpty() || !msg.errorName().isEmpty()) {
            qWarning() << Q_FUNC_INFO << msg.errorMessage();
            Q_EMIT NotifyOfflineUpgradeProgress(30, -1);
        }
    } else {
        Q_EMIT NotifyOfflineUpgradeProgress(50, 1);
        Q_EMIT NotifyExecDistUpgradeInfo("开始进行系统升级...");
        int exitcode = m_offlineCheck->execSystemUpgrade();
        if (0 == exitcode) {
            Q_EMIT NotifyOfflineUpgradeProgress(100, 1);
        } else {
            Q_EMIT NotifyOfflineUpgradeProgress(80, -2);
        }
    }
}

void WriteDBusData::acceptBackupSystemProgress(QString kind, bool success, QString errMsg)
{
    if ("backup" == kind) {
        if (success) { // 执行升级操作
            Q_EMIT NotifyExecDistUpgradeInfo("系统备份完成");
            Q_EMIT NotifyOfflineUpgradeProgress(50, 1);
            Q_EMIT NotifyExecDistUpgradeInfo("开始进行系统升级...");
            int exitcode = m_offlineCheck->execSystemUpgrade();
            if (0 == exitcode) {
                Q_EMIT NotifyOfflineUpgradeProgress(100, 1);
            } else {
                Q_EMIT NotifyOfflineUpgradeProgress(80, -2);
            }
        } else {
            Q_EMIT NotifyOfflineUpgradeProgress(70, -1);
            qWarning() << errMsg;
        }
    }
}

void WriteDBusData::StartUpgradeSystem()
{
    if (!isValidInvoker()) {
        return;
    }

    Q_EMIT NotifyOfflineUpgradeProgress(50, 1);
    Q_EMIT NotifyExecDistUpgradeInfo("开始进行系统升级...");
    int exitcode = m_offlineCheck->execSystemUpgrade();
    if (0 == exitcode) {
        Q_EMIT NotifyOfflineUpgradeProgress(100, 1);
    } else {
        Q_EMIT NotifyOfflineUpgradeProgress(80, -2);
    }
}

void WriteDBusData::StartExecHookPost()
{
    if (!isValidInvoker()) {
        return;
    }

    m_offlineCheck->execHookPost(m_homepath);
}

// 校验调用者
bool WriteDBusData::isValidInvoker()
{
#ifdef QT_DEBUG
    return true;
#else
    // 判断是否位自身调用
    if (!calledFromDBus()) {
        return true;
    }

    bool valid = false;
    QDBusConnection conn = connection();
    QDBusMessage msg = message();

    // 判断是否存在执行路径且是否存在于可调用者名单中
    uint pid = conn.interface()->servicePid(msg.service()).value();
    QFile initNsMntFile("/proc/1/ns/mnt");
    QFile senderNsMntFile(QString("/proc/%1/ns/mnt").arg(pid));
    auto initNsMnt = initNsMntFile.readLink().trimmed().remove(0, QString("/proc/1/ns/mnt").length());
    auto senderNsMnt = senderNsMntFile.readLink().trimmed().remove(0, QString("/proc/%1/ns/mnt").arg(pid).length());
    if (initNsMnt != senderNsMnt) {
        sendErrorReply(QDBusError::ErrorType::Failed, "Illegal calls！！！！！");
        return false;
    }

    //进制使用环境变量导入.so动态调用dbus接口
    QProcess proc;
    proc.start(QString("cat /proc/%1/maps").arg(pid));
    proc.waitForStarted();
    proc.waitForFinished();
    QString maps = QString::fromLocal8Bit(proc.readAllStandardOutput()).trimmed();
    proc.close();
    QStringList mapsList = maps.split(" ", QString::SkipEmptyParts).filter(".so");
    for (int i = 0; i < mapsList.count(); ++i) {
        QStringList libpath = mapsList.at(i).split("/", QString::SkipEmptyParts);
        QString libhead = QString("/%1/%2").arg(libpath.at(0)).arg(libpath.at(1));
        if (libhead != "/usr/lib") {
            sendErrorReply(QDBusError::ErrorType::Failed, "Illegal calls！！！！！");
            return false;
        }
    }

    QFileInfo f(QString("/proc/%1/exe").arg(pid));
    QString invokerPath = f.canonicalFilePath();
    // 防止读取的执行路径为空，为空时取cmdline内容
    if (invokerPath.isEmpty()) {
        QFile cmdlineFile(QString("/proc/%1/cmdline").arg(pid));
        if (cmdlineFile.open(QIODevice::OpenModeFlag::ReadOnly)) {
            QString sCmdline = cmdlineFile.readAll();
            invokerPath = sCmdline.split(" ").first();
        }
        cmdlineFile.close();
    }

    if (ValidInvokerExePathList.contains(invokerPath)) {
        valid = true;
    }

    // 如果是非法调用者，则输出错误
    if (!valid) {
        sendErrorReply(QDBusError::ErrorType::Failed,
                       QString("(pid: %1)[%2] is not allowed to transfer defender inter")
                           .arg(pid)
                           .arg(invokerPath));
    }

    return valid;
#endif
}
