/*
 * Copyright (C) 2023 ~ 2024 UnionTech Software Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "mainwindow.h"
#include "widgets/multiselectlistview.h"

#include "window/modules/common/common.h"
#include "window/modules/common/compixmap.h"
#include "window/modules/offlineupdate/offlineupdatemodule.h"
#include "window/modules/systemupdate/systemupdatemodule.h"
#include "window/modules/isoupdate/isoupdatemodule.h"

#include <DTitlebar>
#include <DBackgroundGroup>
#include <DFloatingMessage>
#include <DMessageManager>

Q_DECLARE_METATYPE(QMargins)

const int WidgetMinimumWidget = 740;
const int WidgetMinimumHeight = 600;

const QMargins navItemMargin(5, 3, 5, 3);
const QVariant NavItemMargin = QVariant::fromValue(navItemMargin);

MainWindow::MainWindow(QWidget *parent)
    : DMainWindow(parent)
    , m_contentLayout(nullptr)
    , m_rightContentLayout(nullptr)
    , m_navView(nullptr)
    , m_rightView(nullptr)
    , m_navModel(nullptr)
    , m_offLineUpdateModule(nullptr)
    , m_pSystemTray(nullptr)
    , m_trayMenu(nullptr)
    , m_trayDialag(nullptr)
    , m_settings(new QGSettings("com.deepin.offline.update", QByteArray(), this))
{
    setObjectName(MAIN_WINDOW_OBJ_NAME);
    // 设置主窗口属性
    setFixedSize(QSize(WidgetMinimumWidget, WidgetMinimumHeight));
    setWindowFlags(Qt::WindowCloseButtonHint);

    // 初始化
    initUI();
    initData();
    //initTray();
    initAllModule();
    modulePreInitialize();

    // 导航栏是否显示
    m_navShowStatus = m_settings->get(NAV_SHOW_TYPE).toBool();
    m_navView->setVisible(m_navShowStatus);
}

MainWindow::~MainWindow()
{
    if (m_navModel) {
        m_navModel->clear();
        m_navModel->deleteLater();
        m_navModel = nullptr;
    }
}

// 初始化
void MainWindow::initUI()
{
    // 标题栏 -logo
    DTitlebar *titlebar = this->titlebar();
    titlebar->setIcon(QIcon::fromTheme(UPGRADE_TOOL));
    // 标题栏 -主题显示状态
    titlebar->setMenuVisible(false);

    // Initialize view and layout structure
    QWidget *content = new QWidget(this);
    setCentralWidget(content);

    m_contentLayout = new QHBoxLayout(content);
    m_contentLayout->setContentsMargins(0, 0, 0, 0);
    m_contentLayout->setMargin(0);
    m_contentLayout->setSpacing(0);
    // 左边list
    m_navView = new MultiSelectListView(this);
    m_navView->setItemSize(QSize(180, 48));
    m_navView->setItemSpacing(0);
    m_navView->setSpacing(20);
    m_navView->setIconSize(QSize(32, 32));
    m_navView->setFrameShape(QFrame::Shape::NoFrame);
    m_navView->setEditTriggers(QListView::NoEditTriggers);
    m_navView->setAutoScroll(true); //滚动条
    m_navView->setTabKeyNavigation(false); //tab按键是否有效
    m_navView->setViewportMargins(QMargins(10, 10, 10, 0)); //滚动区域边框大小
    m_navView->setFixedWidth(200); //固定窗口值
    m_navView->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    m_navView->setSpacing(0);
    m_navView->setSelectionMode(QAbstractItemView::SingleSelection);
    m_navView->setAutoFillBackground(true);
    m_navView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));
    m_contentLayout->addWidget(m_navView);

    // 右侧布局
    m_rightContentLayout = new QHBoxLayout;

    m_rightView = new DBackgroundGroup(m_rightContentLayout);
    m_rightView->setItemSpacing(2);
    m_rightView->setItemMargins(QMargins(0, 0, 0, 0));
    m_rightView->setContentsMargins(0, 0, 0, 0);
    m_rightView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_contentLayout->addWidget(m_rightView);
}

void MainWindow::initData()
{
    // Initialize top page view and model
    m_navModel = new QStandardItemModel(m_navView);
    m_navView->setModel(m_navModel);
    // 连接按下信号与窗口切换槽函数
    connect(m_navView, &DListView::pressed, this, &MainWindow::onFirstItemClick);
}

// 初始化托盘
void MainWindow::initTray()
{
    m_pSystemTray = new QSystemTrayIcon(this);
    m_trayMenu = new QMenu(this);

    // 安全中心
    QAction *show = new QAction(this);
    show->setText("离线升级工具");
    show->setToolTip("离线升级工具");
    m_trayMenu->addAction(show);
    // 点击托盘中的安全中心  激活该窗口并且展现出来
    connect(show, &QAction::triggered, this, [this] {
        this->show();
        this->setWindowState(Qt::WindowState::WindowActive);
    });

    // 托盘退出
    QAction *exit = new QAction(this);
    exit->setText("退出");
    m_trayMenu->addAction(exit);
    connect(exit, &QAction::triggered, this, [=] {
        closeWindow();
    });

    // 设置系统托盘的上下文菜单
    m_pSystemTray->setContextMenu(m_trayMenu);
    // 设置系统托盘提示信息、托盘图标
    m_pSystemTray->setToolTip("离线升级工具");
    m_pSystemTray->setIcon(QIcon::fromTheme("upgrade-tool"));
    connect(m_pSystemTray, &QSystemTrayIcon::activated, this, &MainWindow::onTrayActivated);

    m_pSystemTray->show();
}

void MainWindow::popWidget(ModuleInterface *const inter)
{
    Q_UNUSED(inter)
}

void MainWindow::popAndDelWidget(ModuleInterface *const inter)
{
    Q_UNUSED(inter)
}

void MainWindow::pushWidget(ModuleInterface *const inter, QWidget *const w)
{
    if (!inter) {
        qWarning() << Q_FUNC_INFO << " inter is nullptr";
        return;
    }

    if (!w) {
        qWarning() << Q_FUNC_INFO << " widget is nullptr";
        return;
    }

    pushNormalWidget(inter, w);
}

void MainWindow::setModuleVisible(ModuleInterface *const inter, const bool visible)
{
}

void MainWindow::setCurrentModule(int iModuleIdx, int iPageIdx)
{
    // 获取QModelIndex
    QModelIndex modelIndex = m_navModel->index(iModuleIdx, 0);
    m_navView->setCurrentIndex(m_navModel->index(iModuleIdx, 0));
    // 将显示栈清空
    popAllWidgets(0);
    this->setFocus();

    // 激活模块
    ModuleInterface *inter = m_modules[modelIndex.row()].first;
    inter->active(iPageIdx);
}

ModuleInterface *MainWindow::getCurrentModule()
{
    if (m_contentStack.isEmpty()) {
        return nullptr;
    }

    return m_contentStack.top().first;
}

int MainWindow::getModuleIndex(const QString &name)
{
    int index = 0;
    for (QPair<ModuleInterface *, QString> p : m_modules) {
        if (p.first->name() == name)
            return index;
        index++;
    }
    return -1;
}

//初始化模块
void MainWindow::initAllModule()
{
    m_offLineUpdateModule = new OfflineUpdateModule(this, this);
    connect(m_offLineUpdateModule, &OfflineUpdateModule::sendToolExit, this, &MainWindow::closeWindow);
    m_modules = {
        {m_offLineUpdateModule, "离线更新"},
        {new ISOUpdateModule(this, this), "iso更新"},
        {new SystemUpdateModule(this, this), "系统更新"},
    };

    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
        DStandardItem *item = new DStandardItem;
        item->setAccessibleText(it->first->name());
        item->setText(it->second);
        item->setIcon(QIcon::fromTheme("upgrade-tool"));

        item->setData(NavItemMargin, Dtk::MarginsRole);
        item->setData(QVariant(it->first->name()), Qt::AccessibleTextRole);

        m_navModel->appendRow(item);
    }
}

//模块初始化前准备
void MainWindow::modulePreInitialize()
{
    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
        QElapsedTimer et;
        et.start();
        it->first->preInitialize();
    }

    QModelIndex index0 = m_navModel->index(0, 0);
    if (index0.isValid()) {
        onFirstItemClick(index0); //默认点击首页
    }
}

//左侧列表的点击
void MainWindow::onFirstItemClick(const QModelIndex &index)
{
    m_navView->setCurrentIndex(index);
    ModuleInterface *inter = m_modules[index.row()].first;

    //有数据  并且最后一个和现在点击的一致
    if (!m_contentStack.isEmpty() && m_contentStack.last().first == inter) {
        return;
    }
    popAllWidgets(0);

    inter->active(0);
    m_navView->resetStatus(index);
    m_navView->setFocus();
}

// 功能模块切换出栈
void MainWindow::popAllWidgets(int place)
{
    Q_UNUSED(place);
    if (!m_contentStack.count())
        return;

    popWidget();
    // 清除栈容器内所有对象 --确保外部接口调用菜单显示之前栈容器为空
    m_contentStack.clear();
}

// 模块出栈
void MainWindow::popWidget()
{
    if (!m_contentStack.size())
        return;

    // 反激活切换前的模块 - widget是否需要delete，由各个模块的module自己决定
    ModuleInterface *inter = getCurrentModule();

    // 掩藏/移除切换前的模块
    QWidget *w = m_contentStack.pop().second; //栈的顶栈的widget
    m_rightContentLayout->removeWidget(w);

    inter->deactive();
}

// 模块入栈
void MainWindow::pushNormalWidget(ModuleInterface *const inter, QWidget *const w)
{
    //Set the newly added page to fill the blank area
    w->setParent(this);
    w->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    w->show();

    m_contentStack.push({inter, w});
    m_rightContentLayout->addWidget(w, m_contentStack.size() == 1 ? 3 : 7);
}

void MainWindow::showModulePage(const QString &module, const QString &page)
{
    int moduleIndex = 0;
    int pageIndex = 0;
    ModuleInterface *inter = nullptr;
    for (int i = 0; i < m_modules.count(); i++) {
        inter = m_modules[i].first;
        if (module == inter->name()) {
            moduleIndex = i;
            break;
        }
    }

    // 判断子页面索引
    pageIndex = getPageIndex(page);

    setCurrentModule(moduleIndex, pageIndex);
    // 显示并恢复窗口为正常状态
    showNormal();
    activateWindow();
}

// 寻找子页面索引
int MainWindow::getPageIndex(QString pageName)
{
    int retIndex = 0;

    return retIndex;
}

int MainWindow::getModuleCount()
{
    return m_modules.count();
}

void MainWindow::onTrayActivated(QSystemTrayIcon::ActivationReason state)
{
    if (state == QSystemTrayIcon::ActivationReason::Trigger) {
        if (!this->isVisible()) {
            //页面隐藏，点击托盘重新show
            this->show();
        } else {
            //页面能显示的时候，点击托盘会＂正常/最小化＂切换窗口
            if (this->isMinimized()) {
                this->setWindowState(Qt::WindowState::WindowActive);
            } else {
                this->setWindowState(Qt::WindowState::WindowMinimized);
            }
        }
    }
}

// 关闭窗口事件
void MainWindow::closeEvent(QCloseEvent *event)
{
    Q_UNUSED(event)
    closeWindow();
}

void MainWindow::closeWindow()
{
    qApp->quit();
}

void MainWindow::setISOUpdateFile(QString filepath)
{
    Q_UNUSED(filepath)
    ModuleInterface *inter = getCurrentModule();
    if (inter->name() != "isoupdate") {
        showModulePage("isoupdate", "");
    }
    m_navView->setVisible(m_navShowStatus);
}

void MainWindow::setOfflineUpdateFile(QString filepath)
{
    ModuleInterface *inter = getCurrentModule();
    if (inter->name() != "offlineupdate") {
        showModulePage("offlineupdate", "");
    }
    m_offLineUpdateModule->setOfflineUpdateFile(filepath);
    m_navView->setVisible(m_navShowStatus);
}

void MainWindow::showErrorFileType()
{
    DFloatingMessage *floMsgSwitchOff = new DFloatingMessage(DFloatingMessage::TransientType);
    floMsgSwitchOff->setDuration(2000);
    floMsgSwitchOff->setIcon(QIcon::fromTheme(DIALOG_TIP_YELLOW));
    floMsgSwitchOff->setMessage(tr("File format error, please reselect"));
    DMessageManager::instance()->sendMessage(this, floMsgSwitchOff);
    DMessageManager::instance()->setContentMargens(this, QMargins(0, 0, 0, 10));
}
