/*
 * Copyright (C) 2023 ~ 2024 UnionTech Software Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "offlinecheckresultwidget.h"
#include "offlineupgradedialog.h"
#include "offlineupdatemodel.h"
#include "licensedialog.h"
#include "confirmdialog.h"
#include "window/modules/common/common.h"
#include "window/modules/common/compixmap.h"

#include <DFloatingMessage>
#include <DMessageManager>

#include <QVBoxLayout>
#include <QPalette>
#include <QColor>
#include <QFileDialog>
#include <QCheckBox>
#include <QTimer>

OfflineCheckResultWidget::OfflineCheckResultWidget(OfflineUpdateModel *model, int itemType, bool result, DWidget *parent)
    : DWidget(parent)
    , m_model(model)
    , m_itemType(itemType)
    , m_result(result)
    , m_errorContent(nullptr)
    , m_errorLabel(nullptr)
    , m_resultContent(nullptr)
    , m_tableView(nullptr)
    , m_tableModel(nullptr)
    , m_headerDelegate(nullptr)
    , m_offlineUpdateDelegate(nullptr)
    , m_checkBox(nullptr)
    , m_recoverSystemBtn(nullptr)
    , m_backSystemCheckBox(nullptr)
    , m_logLabel(nullptr)
    , m_logPathLabel(nullptr)
    , m_upgradeBtn(nullptr)
    , m_offlineUpgradeDialog(nullptr)
{
    // 初始化
    initUI();
    setCheckResultWidget(m_itemType, m_result);
    connect(m_model, &OfflineUpdateModel::notifyOfflineUpgradeProgress, this, &OfflineCheckResultWidget::acceptOfflineUpgradeProgress);
    connect(m_model, &OfflineUpdateModel::notifyExecDistUpgradeInfo, this, &OfflineCheckResultWidget::acceptExecDistUpgradeInfo);
}

OfflineCheckResultWidget::~OfflineCheckResultWidget()
{
}

// 初始化界面
void OfflineCheckResultWidget::initUI()
{
    // 主布局
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(0);
    this->setLayout(mainLayout);

    QSpacerItem *spacer1 = new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);
    mainLayout->addItem(spacer1);
    // 错误界面加载
    mainLayout->addWidget(loadCheckErrorWidget());

    // 结果界面加载
    mainLayout->addWidget(loadCheckResultWidget());
    QSpacerItem *spacer2 = new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);
    mainLayout->addItem(spacer2);

    m_checkBox->setEnabled(false);
    m_recoverSystemBtn->setEnabled(false);
    m_backSystemCheckBox->setEnabled(false);
    m_upgradeBtn->setEnabled(false);
}

QWidget *OfflineCheckResultWidget::loadCheckErrorWidget()
{
    m_errorContent = new QWidget(this);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(10);
    m_errorContent->setLayout(layout);

    // 错误图标
    QLabel *errorPic = new QLabel(this);
    errorPic->setAlignment(Qt::AlignHCenter);
    errorPic->setPixmap(QIcon::fromTheme("icon_fail").pixmap(100, 100));
    layout->addWidget(errorPic, 0, Qt::AlignCenter);

    // 错误说明
    m_errorLabel = new DLabel(this);
    m_errorLabel->setText(tr("Insufficient disk space, please clean up disk space and try again!"));
    DFontSizeManager::instance()->bind(m_errorLabel, DFontSizeManager::T6);
    layout->addWidget(m_errorLabel, 0, Qt::AlignCenter);

    m_errorContent->setLayout(layout);
    return m_errorContent;
}

QWidget *OfflineCheckResultWidget::loadCheckResultWidget()
{
    m_resultContent = new QWidget(this);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(10, 10, 10, 10);
    layout->setSpacing(0);
    m_resultContent->setLayout(layout);

    // 加载表格空间
    layout->addWidget(loadOfflineUpdateTable());

    layout->addSpacing(10);
    // 勾选确认
    QHBoxLayout *tipLayout = new QHBoxLayout;
    tipLayout->setContentsMargins(0, 0, 0, 0);
    tipLayout->setSpacing(0);
    // 勾选框
    m_checkBox = new QCheckBox(this);
    m_checkBox->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_checkBox->setText(tr("I have read and understand"));
    DFontSizeManager::instance()->bind(m_checkBox, DFontSizeManager::T6);
    tipLayout->addWidget(m_checkBox, 0, Qt::AlignLeft);
    connect(m_checkBox, &QCheckBox::stateChanged, this, &OfflineCheckResultWidget::checkBoxChange);
    // 恢复系统说明
    QString recoverInfo = tr("How to restore the system after abnormal system startup after successful upgrade");
    m_recoverSystemBtn = new DCommandLinkButton("", this);
    QFontMetrics fontWidth(m_recoverSystemBtn->font());
    QString elideNote = fontWidth.elidedText(recoverInfo, Qt::ElideMiddle, 490);
    m_recoverSystemBtn->setText(elideNote);
    m_recoverSystemBtn->setToolTip(recoverInfo);
    DFontSizeManager::instance()->bind(m_recoverSystemBtn, DFontSizeManager::T6);
    m_recoverSystemBtn->installEventFilter(this);
    tipLayout->addWidget(m_recoverSystemBtn, 0, Qt::AlignLeft);
    tipLayout->addStretch();
    layout->addLayout(tipLayout);
    connect(m_recoverSystemBtn, &DCommandLinkButton::clicked, this, [=] {
        LicenseDialog *dialog = new LicenseDialog(tr("How to restore the system after abnormal system startup after successful upgrade"), this);
        connect(dialog, &LicenseDialog::close, this, [=] {
            dialog->deleteLater();
        });
        dialog->exec();
    });

    m_backSystemCheckBox = new QCheckBox(this);
    m_backSystemCheckBox->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_backSystemCheckBox->setText(tr("Upgrade after the system is backed up"));
    DFontSizeManager::instance()->bind(m_backSystemCheckBox, DFontSizeManager::T6);
    layout->addWidget(m_backSystemCheckBox, 0, Qt::AlignLeft);
    m_backSystemCheckBox->setChecked(true);
    connect(m_backSystemCheckBox, &QCheckBox::stateChanged, this, &OfflineCheckResultWidget::backSystemCheckBoxChange);

    layout->addSpacing(50);
    QHBoxLayout *logLayout = new QHBoxLayout;
    logLayout->setContentsMargins(0, 0, 0, 0);
    logLayout->setSpacing(0);
    logLayout->addStretch();
    // 日志标签
    m_logLabel = new DLabel(this);
    m_logLabel->setText(tr("The update log has been saved on the desktop:"));
    DFontSizeManager::instance()->bind(m_logLabel, DFontSizeManager::T7);
    logLayout->addWidget(m_logLabel, 0, Qt::AlignHCenter);

    m_logPathLabel = new DTipLabel("", this);
    m_logPathLabel->setText(QString("%1/Desktop/离线oup升级日志.log").arg(QDir::homePath()));
    DFontSizeManager::instance()->bind(m_logPathLabel, DFontSizeManager::T7);
    logLayout->addWidget(m_logPathLabel, 0, Qt::AlignHCenter);

    logLayout->addStretch();
    layout->addLayout(logLayout);

    layout->addSpacing(10);
    // 开始升级按钮
    m_upgradeBtn = new DSuggestButton(this);
    m_upgradeBtn->setText(tr("Start Upgrade"));
    m_upgradeBtn->setFixedWidth(200);
    layout->addWidget(m_upgradeBtn, 0, Qt::AlignCenter);
    connect(m_upgradeBtn, &QCheckBox::clicked, this, &OfflineCheckResultWidget::startCVEUpgrade);

    m_resultContent->setLayout(layout);
    return m_resultContent;
}

QWidget *OfflineCheckResultWidget::loadOfflineUpdateTable()
{
    //添加圆角外边框
    DFrame *frame = new DFrame;
    frame->setFixedHeight(330);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(1, 1, 1, 1);
    frame->setLayout(layout);

    // 表格
    m_tableView = new DTableView(this);
    m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableView->setSelectionMode(QAbstractItemView::NoSelection);
    m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableView->setFrameShape(QTableView::NoFrame);
    m_tableView->horizontalHeader()->setStretchLastSection(true);
    m_tableView->verticalHeader()->setHidden(true);
    m_tableView->setShowGrid(false);
    m_tableView->verticalHeader()->setDefaultSectionSize(48);
    layout->addWidget(m_tableView);

    // 设置代理
    m_offlineUpdateDelegate = new OfflineUpdateDelegate(this);
    m_tableView->setItemDelegate(m_offlineUpdateDelegate);

    // 更换表头
    m_headerDelegate = new DefenderTableHeaderView(Qt::Horizontal, this);
    m_headerDelegate->setFixedHeight(36);
    m_tableView->setHorizontalHeader(m_headerDelegate);

    // 设置表头
    m_tableModel = new QStandardItemModel(this);
    m_tableModel->setParent(this);
    m_tableModel->setHorizontalHeaderItem(TABLE_COLUMN_CHECKITEM, new QStandardItem(tr("Detection Item")));
    m_tableModel->setHorizontalHeaderItem(TABLE_COLUMN_CHECKRESULT, new QStandardItem(tr("Result")));
    m_tableModel->setHorizontalHeaderItem(TABLE_COLUMN_CHECKVALUE, new QStandardItem("resultvalue"));

    // 表格-模型
    m_tableView->setModel(m_tableModel);
    // 屏蔽排序/表头排序图标
    m_tableView->setSortingEnabled(false);
    m_tableView->horizontalHeader()->setSortIndicatorShown(false);

    //设置各栏宽度
    m_tableView->setColumnWidth(TABLE_COLUMN_CHECKITEM, 300);
    m_tableView->setColumnWidth(TABLE_COLUMN_CHECKRESULT, 150);

    m_tableView->setColumnHidden(TABLE_COLUMN_CHECKVALUE, true);
    return frame;
}

// 设置结果页显示空间
void OfflineCheckResultWidget::setCheckResultWidget(int itemType, bool result)
{
    if (OfflineCheckItem::OfflineFileArXSpace == itemType && !result) {
        m_resultContent->setVisible(false);
        m_errorLabel->setText(tr("Insufficient disk space, please clean up disk space and try again!"));
    } else if (OfflineCheckItem::OfflineFileDirComplete == itemType && !result) {
        m_resultContent->setVisible(false);
        m_errorLabel->setText(tr("File corrupted, please check and try again!"));
    } else if (OfflineCheckItem::HooksPre == itemType && !result) {
        m_resultContent->setVisible(false);
        m_errorLabel->setText(tr("System environment is abnormal, contact the administrator！"));
    } else {
        m_errorContent->setVisible(false);

        QList<CheckResultInfo> checkItemMap;
        if (UpgradeType::CVEPatchUpgrade == m_model->getUpgradeType()) {
            CheckResultInfo info1;
            info1.name = tr("Upgrade Type");
            info1.resultinfo = tr("CVE Patch Update");
            info1.resultvalue = -1;
            checkItemMap.append(info1);

            CheckResultInfo info2;
            info2.name = tr("Vulnerability ID");
            info2.resultinfo = m_model->getPatchUpgradeSerial();
            info2.resultvalue = -1;
            checkItemMap.append(info2);
        } else {
            CheckResultInfo info1;
            info1.name = tr("Upgrade Type");
            info1.resultinfo = tr("System Update");
            info1.resultvalue = -1;
            checkItemMap.append(info1);
        }

        if (OfflineCheckItem::SystemVersion == itemType) {
            CheckResultInfo info3;
            info3.name = tr("System Version Detection");
            info3.resultinfo = tr("Fail");
            info3.resultvalue = CHECK_RESULT_NOPASS;
            checkItemMap.append(info3);

            CheckResultInfo info4;
            info4.name = tr("Offline Package Architecture Detection");
            info4.resultinfo = tr("Not Detected");
            info4.resultvalue = CHECK_RESULT_NO;
            checkItemMap.append(info4);

            CheckResultInfo info5;
            info5.name = tr("Disk Space Detection");
            info5.resultinfo = tr("Not Detected");
            info5.resultvalue = CHECK_RESULT_NO;
            checkItemMap.append(info5);

            CheckResultInfo info6;
            info6.name = tr("Offline Package Update Number Detection");
            info6.resultinfo = tr("Not Detected");
            info6.resultvalue = CHECK_RESULT_NO;
            checkItemMap.append(info6);
        } else if (OfflineCheckItem::OfflineFileArch == itemType) {
            CheckResultInfo info3;
            info3.name = tr("System Version Detection");
            info3.resultinfo = tr("Pass");
            info3.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info3);

            CheckResultInfo info4;
            info4.name = tr("Offline Package Architecture Detection");
            info4.resultinfo = tr("Fail");
            info4.resultvalue = CHECK_RESULT_NOPASS;
            checkItemMap.append(info4);

            CheckResultInfo info5;
            info5.name = tr("Disk Space Detection");
            info5.resultinfo = tr("Not Detected");
            info5.resultvalue = CHECK_RESULT_NO;
            checkItemMap.append(info5);

            CheckResultInfo info6;
            info6.name = tr("Offline Package Update Number Detection");
            info6.resultinfo = tr("Not Detected");
            info6.resultvalue = CHECK_RESULT_NO;
            checkItemMap.append(info6);
        } else if (OfflineCheckItem::SystemBackupSpace == itemType) {
            CheckResultInfo info3;
            info3.name = tr("System Version Detection");
            info3.resultinfo = tr("Pass");
            info3.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info3);

            CheckResultInfo info4;
            info4.name = tr("Offline Package Architecture Detection");
            info4.resultinfo = tr("Pass");
            info4.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info4);

            CheckResultInfo info5;
            info5.name = tr("Disk Space Detection");
            info5.resultinfo = tr("Fail");
            info5.resultvalue = CHECK_RESULT_NOPASS;
            checkItemMap.append(info5);

            CheckResultInfo info6;
            info6.name = tr("Offline Package Update Number Detection");
            info6.resultinfo = tr("Not Detected");
            info6.resultvalue = CHECK_RESULT_NO;
            checkItemMap.append(info6);
        } else if (OfflineCheckItem::UpgradeSum == itemType && !result) {
            CheckResultInfo info3;
            info3.name = tr("System Version Detection");
            info3.resultinfo = tr("Pass");
            info3.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info3);

            CheckResultInfo info4;
            info4.name = tr("Offline Package Architecture Detection");
            info4.resultinfo = tr("Pass");
            info4.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info4);

            CheckResultInfo info5;
            info5.name = tr("Disk Space Detection");
            info5.resultinfo = tr("Pass");
            info5.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info5);

            CheckResultInfo info6;
            info6.name = tr("Offline Package Update Number Detection");
            info6.resultinfo = tr("No available update package");
            info6.resultvalue = CHECK_RESULT_NOPASS;
            checkItemMap.append(info6);
        } else {
            CheckResultInfo info3;
            info3.name = tr("System Version Detection");
            info3.resultinfo = tr("Pass");
            info3.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info3);

            CheckResultInfo info4;
            info4.name = tr("Offline Package Architecture Detection");
            info4.resultinfo = tr("Pass");
            info4.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info4);

            CheckResultInfo info5;
            info5.name = tr("Disk Space Detection");
            info5.resultinfo = tr("Pass");
            info5.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info5);

            CheckResultInfo info6;
            info6.name = tr("Offline Package Update Number Detection");
            info6.resultinfo = tr("%1 item(s)").arg(m_model->getDistUpgradeSum());
            info6.resultvalue = CHECK_RESULT_PASS;
            checkItemMap.append(info6);

            m_checkBox->setEnabled(true);
            m_recoverSystemBtn->setEnabled(true);
        }
        for (int i = 0; i < checkItemMap.count(); ++i) {
            CheckResultInfo info = checkItemMap.at(i);
            QString name = info.name;
            QString result = info.resultinfo;
            int resultvalue = info.resultvalue;
            m_tableModel->setItem(i, 0, new QStandardItem(name));
            m_tableModel->setItem(i, 1, new QStandardItem(result));
            m_tableModel->setItem(i, 2, new QStandardItem(QString::number(resultvalue)));
        }
    }
}

// 选择框的改变
void OfflineCheckResultWidget::checkBoxChange(int status)
{
    m_upgradeBtn->setEnabled(status);
    m_backSystemCheckBox->setEnabled(status);
}

// 系统备份勾选框改变
void OfflineCheckResultWidget::backSystemCheckBoxChange(int status)
{
    if (!status) {
        DDialog *dialog = new DDialog(this);
        dialog->setFixedSize(QSize(380, 145));
        dialog->setIcon(QIcon::fromTheme(DIALOG_TIP_YELLOW));
        dialog->setMessage(tr("Without backing up the system, an upgrade failure will prevent normal rollback. Are you sure you want to proceed with the upgrade without backing up the system?"));
        dialog->setWordWrapMessage(true);
        dialog->setAttribute(Qt::WA_DeleteOnClose);

        DWarningButton *cancelBtn = new DWarningButton(dialog);
        cancelBtn->setText(tr("Cancel", "button"));
        cancelBtn->setAttribute(Qt::WA_NoMousePropagation);
        dialog->insertButton(0, cancelBtn, true);
        QPushButton *confirmBtn = new QPushButton(dialog);
        confirmBtn->setText(tr("Confirm", "button"));
        confirmBtn->setAttribute(Qt::WA_NoMousePropagation);
        dialog->insertButton(1, confirmBtn, false);

        connect(dialog, &DDialog::buttonClicked, this, [=](int index, QString) {
            switch (index) {
            case 0:
                m_backSystemCheckBox->setCheckState(Qt::Checked);
                dialog->close();
                break;
            case 1:
                dialog->accept();
                break;
            default:
                break;
            }
        });
        // 关闭时候设置扫描中退出的提示框状态
        connect(dialog, &DDialog::aboutToClose, this, [=]() {
            m_backSystemCheckBox->setCheckState(Qt::Checked);
        });
        // 弹窗为阻态
        dialog->exec();
    }
}
// 开始升级
void OfflineCheckResultWidget::startCVEUpgrade()
{
    ConfirmDialog *dialog = new ConfirmDialog(m_model->getABSystemBackupStatus(), this);
    connect(dialog, &ConfirmDialog::sendStartUpgrade, this, [=] {
        // 开始升级弹框
        m_offlineUpgradeDialog = new OfflineUpgradeDialog(m_model, this);
        m_offlineUpgradeDialog->raise();
        m_offlineUpgradeDialog->show();
        m_offlineUpgradeDialog->grabKeyboard();

        // 开始升级
        m_model->startOfflineUpgrade(m_backSystemCheckBox->isChecked());
    });
    connect(dialog, &LicenseDialog::close, this, [=] {
        dialog->deleteLater();
    });
    dialog->exec();
}

void OfflineCheckResultWidget::acceptOfflineUpgradeProgress(int progress, int code)
{
    if (code < 0) {
        m_offlineUpgradeDialog->releaseKeyboard();
        m_offlineUpgradeDialog->close();
        QString error;
        if (-1 == code) {
            error = tr("System backup failed, please contact administrator to resolve.");
        } else {
            error = tr("The upgrade fails due to an exception in the resource package, perform a system restoration.");
        }

        QTimer::singleShot(1000, this, [=] {
            DDialog dialog;
            dialog.setFixedSize(QSize(300, 150));
            dialog.setIcon(QIcon::fromTheme(DIALOG_TIP_YELLOW));
            dialog.setModal(true);
            dialog.setMessage(error);
            dialog.setWordWrapMessage(true);
            dialog.addButton(tr("OK", "Button"), false);
            dialog.exec();
        });
        return;
    }

    if (m_offlineUpgradeDialog) {
        m_offlineUpgradeDialog->setUpgradeProgress(progress);
    }
    if (100 == progress) {
        m_offlineUpgradeDialog->releaseKeyboard();
        m_offlineUpgradeDialog->close();
        m_model->systemReboot();

        DFloatingMessage *floMsgSwitchOff = new DFloatingMessage(DFloatingMessage::TransientType);
        floMsgSwitchOff->setDuration(2000);
        floMsgSwitchOff->setIcon(QIcon::fromTheme(DIALOG_TIP_YELLOW));
        floMsgSwitchOff->setMessage(tr("If the upgrade is successful, restart the system to take effect"));
        DMessageManager::instance()->sendMessage(this, floMsgSwitchOff);
        DMessageManager::instance()->setContentMargens(this, QMargins(0, 0, 0, 10));
    }
}

void OfflineCheckResultWidget::acceptExecDistUpgradeInfo(QString info)
{
    if (m_offlineUpgradeDialog && !info.isEmpty()) {
        m_offlineUpgradeDialog->setUpgradeInfo(info.simplified());
    }
}

// 事件过滤器
bool OfflineCheckResultWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::Type::Enter) {
        setCursor(Qt::PointingHandCursor);
    } else if (event->type() == QEvent::Type::Leave) {
        setCursor(Qt::ArrowCursor);
    }

    return QWidget::eventFilter(obj, event);
}
