/*
 * Copyright (C) 2023 ~ 2024 UnionTech Software Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "offlineupdatemodel.h"
#include "window/modules/common/invokers/invokerfactory.h"
#include "window/modules/common/common.h"

#include <QDir>
#include <QDebug>

OfflineUpdateModel::OfflineUpdateModel(QObject *parent)
    : QObject(parent)
    , m_systemUpgradeDBusInter(nullptr)
    , m_wmDBusInter(nullptr)
    , m_rebootDBusInter(nullptr)
{
    // 初始化数据
    initData();
}

OfflineUpdateModel::~OfflineUpdateModel()
{
}

void OfflineUpdateModel::initData()
{
    // 系统更新dbus接口
    m_systemUpgradeDBusInter = InvokerFactory::GetInstance().CreateInvoker("com.deepin.offline.update.daemon",
                                                                           "/com/deepin/offline/update/daemon",
                                                                           "com.deepin.offline.update.daemon",
                                                                           ConnectType::SYSTEM, this);
    m_systemUpgradeDBusInter->Connect("NotifyOfflineFileCheckResult", this, SLOT(acceptOfflineFileCheckResult(int, bool)));
    m_systemUpgradeDBusInter->Connect("NotifyOfflineUpgradeProgress", this, SLOT(acceptOfflineUpgradeProgress(int, int)));
    m_systemUpgradeDBusInter->Connect("NotifyExecDistUpgradeInfo", this, SLOT(acceptExecDistUpgradeInfo(QString)));
    m_systemUpgradeDBusInter->Connect("NotifyOfflineUpgradeLog", this, SLOT(acceptOfflineUpgradeLog(QString)));

    // 获取桌面壁纸dbus接口
    m_wmDBusInter = InvokerFactory::GetInstance().CreateInvoker("com.deepin.wm",
                                                                "/com/deepin/wm",
                                                                "com.deepin.wm",
                                                                ConnectType::SESSION, this);

    m_rebootDBusInter = InvokerFactory::GetInstance().CreateInvoker("com.deepin.SessionManager",
                                                                    "/com/deepin/SessionManager",
                                                                    "com.deepin.SessionManager",
                                                                    ConnectType::SESSION, this);
}

// 设置检测文件路径
void OfflineUpdateModel::setCheckFilePath(const QString &filepath)
{
    Q_EMIT notifyCheckFilePath(filepath);
}

// 开始离线包检测
void OfflineUpdateModel::startOfflineFileChecking(const QString &filepath)
{
    DBUS_NOBLOCK_INVOKE(m_systemUpgradeDBusInter, "StartOfflineFileCheck", QDir::homePath(), filepath);
}
// 离线检测结果
void OfflineUpdateModel::acceptOfflineFileCheckResult(int itemType, bool result)
{
    qInfo() << Q_FUNC_INFO << QString("Offline file check result, itemtype:%1, result:%2").arg(itemType).arg(result);

    if (OfflineCheckItem::CancelUpdate == itemType) {
        Q_EMIT notifyToolExit();
    } else {
        Q_EMIT notifyOfflineCheckResult(itemType, result);
    }
}
// 获取升级类型 系统仓库升级/cve漏洞升级
int OfflineUpdateModel::getUpgradeType()
{
    QDBusMessage msg = DBUS_BLOCK_INVOKE(m_systemUpgradeDBusInter, "GetUpgradeType");
    GET_MESSAGE_VALUE(int, type, msg);
    return type;
}
// 获取cve漏洞编号
QString OfflineUpdateModel::getPatchUpgradeSerial()
{
    QDBusMessage msg = DBUS_BLOCK_INVOKE(m_systemUpgradeDBusInter, "GetPatchUpgradeSerial");
    GET_MESSAGE_VALUE(QString, serail, msg);
    return serail;
}
// 获取更新数量
int OfflineUpdateModel::getDistUpgradeSum()
{
    QDBusMessage msg = DBUS_BLOCK_INVOKE(m_systemUpgradeDBusInter, "GetDistUpgradeSum");
    GET_MESSAGE_VALUE(int, sum, msg);
    return sum;
}

// 获取桌面壁纸
QString OfflineUpdateModel::getDesktopImage()
{
    QDBusMessage msg = DBUS_BLOCK_INVOKE(m_wmDBusInter, "GetCurrentWorkspaceBackground");
    GET_MESSAGE_VALUE(QString, desktopName, msg);
    QStringList desktopList = desktopName.split("://");
    if (desktopList.count() >= 2) {
        return desktopList.at(1);
    } else {
        return "";
    }
}

// ab系统备份是否允许进行备份
bool OfflineUpdateModel::getABSystemBackupStatus()
{
    bool isFlag = false;
    QDBusMessage msg = DBUS_BLOCK_INVOKE(m_systemUpgradeDBusInter, "IsAllowBackupSystem");
    if (!msg.errorMessage().isEmpty() || !msg.errorName().isEmpty()) {
        qWarning() << Q_FUNC_INFO << msg.errorMessage();
    } else {
        GET_MESSAGE_VALUE(bool, status, msg);
        isFlag = status;
    }

    return isFlag;
}

// 开始升级
void OfflineUpdateModel::startOfflineUpgrade(bool canBackup)
{
    if (canBackup) {
        DBUS_NOBLOCK_INVOKE(m_systemUpgradeDBusInter, "BackupSystem");
    } else {
        DBUS_NOBLOCK_INVOKE(m_systemUpgradeDBusInter, "StartUpgradeSystem");
    }
}
// 接受系统升级进度
void OfflineUpdateModel::acceptOfflineUpgradeProgress(int progress, int code)
{
    Q_EMIT notifyOfflineUpgradeProgress(progress, code);
}
void OfflineUpdateModel::acceptExecDistUpgradeInfo(QString info)
{
    Q_EMIT notifyExecDistUpgradeInfo(info);

    QString logPath = QString("%1/Desktop/离线oup升级日志.log").arg(QDir::homePath());
    QFile file(logPath);
    if (file.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {
        QTextStream ts(&file);
        ts << info.simplified() << endl;
    }
    file.close();
}

void OfflineUpdateModel::acceptOfflineUpgradeLog(QString log)
{
    QString logPath = QString("%1/Desktop/离线oup升级日志.log").arg(QDir::homePath());
    QFile file(logPath);
    if (file.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {
        QTextStream ts(&file);
        ts << log << endl;
    }
    file.close();
}

// 重启
void OfflineUpdateModel::systemReboot()
{
    DBUS_BLOCK_INVOKE(m_systemUpgradeDBusInter, "StartExecHookPost", QDir::homePath());
    DBUS_NOBLOCK_INVOKE(m_rebootDBusInter, "RequestReboot");
}
