/*
 * Copyright (C) 2023 ~ 2024 UnionTech Software Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "offlineupgradedialog.h"
#include "offlineupdatemodel.h"
#include "window/modules/common/compixmap.h"
#include "widgets/upgradewidget.h"

#include <DFontSizeManager>
#include <DPalette>

#include <QVBoxLayout>
#include <QLabel>
#include <QDesktopWidget>
#include <QGraphicsOpacityEffect>
#include <QTimer>
#include <QWindow>
#include <QApplication>
#include <QProcess>

OfflineUpgradeDialog::OfflineUpgradeDialog(OfflineUpdateModel *model, QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_progressBar(nullptr)
    , m_progressLabel(nullptr)
    , m_tipLabel(nullptr)
    , m_textEdit(nullptr)
    , m_blurEffect(nullptr)
    , m_upgradeSec(0)
    , m_timer(nullptr)
{
    // 设置为Dialog
    this->setWindowFlags(Qt::X11BypassWindowManagerHint | Qt::Dialog);
    create();
    setAttribute(Qt::WA_NativeWindow);
    windowHandle();
    windowHandle()->setProperty("_d_dwayland_window-type", "launcher");

    // 初始化界面
    initUI();
    // 全屏
    this->showFullScreen();

    // 初始化数据
    initData();

    // 安装定时器
    m_timer = new QTimer();
    connect(m_timer, &QTimer::timeout, this, [=]() {
        QProcess proc;
        proc.start("xdotool key 168");
        proc.waitForStarted();
        proc.waitForFinished();
        proc.close();
    });
    m_timer->start(30 * 1000);
}

OfflineUpgradeDialog::~OfflineUpgradeDialog()
{
    m_timer->stop();
    m_timer->deleteLater();
}

// 布局
void OfflineUpgradeDialog::initUI()
{
    // 底层空间
    QWidget *lowerWidget = new QWidget(this);
    lowerWidget->setGeometry(QRect(0, 0, QApplication::desktop()->width(), QApplication::desktop()->height()));
    lowerWidget->setAutoFillBackground(true);
    QPalette lowerPalette = lowerWidget->palette();
    lowerPalette.setBrush(QPalette::Window,
                          QBrush(QPixmap(m_model->getDesktopImage()).scaled(QSize(QApplication::desktop()->width(), QApplication::desktop()->height()), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
    lowerWidget->setPalette(lowerPalette);

    // 上层空间
    UpgradeWidget *raiseWidget = new UpgradeWidget(QColor::fromRgb(0, 15, 39, int(0.7 * 255)), this);
    raiseWidget->setGeometry(QRect(0, 0, QApplication::desktop()->width(), QApplication::desktop()->height()));

    // 上层控件布局
    QVBoxLayout *mainLayout = new QVBoxLayout(raiseWidget);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(0);

    mainLayout->addStretch();
    // 统信logo
    QLabel *logoPic = new QLabel(raiseWidget);
    logoPic->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    logoPic->setAlignment(Qt::AlignHCenter);
    logoPic->setPixmap(QIcon::fromTheme(":/icons/deepin/builtin/icons/logo.png").pixmap(290, 60));
    mainLayout->addWidget(logoPic, 0, Qt::AlignCenter);

    mainLayout->addSpacing(50);
    // 进度条
    QHBoxLayout *progressLayout = new QHBoxLayout(raiseWidget);
    progressLayout->setContentsMargins(0, 0, 0, 0);
    progressLayout->setSpacing(10);
    progressLayout->addStretch();
    m_progressBar = new QProgressBar(raiseWidget);
    m_progressBar->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_progressBar->setContentsMargins(0, 0, 0, 0);
    m_progressBar->setFixedSize(QSize(500, 8));
    m_progressBar->setAutoFillBackground(false);
    QPalette pal = m_progressBar->palette();
    pal.setColor(QPalette::Window, Qt::white);
    m_progressBar->setPalette(pal);
    progressLayout->addWidget(m_progressBar, 0, Qt::AlignCenter);
    // 进度条百分比显示
    m_progressLabel = new DLabel(raiseWidget);
    m_progressLabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_progressLabel->setText("0%");
    QFont fontProgress = m_progressLabel->font();
    fontProgress.setPixelSize(17);
    DFontSizeManager::instance()->unbind(m_progressLabel);
    QPalette palProgress = m_progressLabel->palette();
    palProgress.setColor(QPalette::WindowText, Qt::white);
    m_progressLabel->setPalette(palProgress);
    progressLayout->addWidget(m_progressLabel, 0, Qt::AlignCenter);
    progressLayout->addStretch();
    mainLayout->addLayout(progressLayout);

    mainLayout->addSpacing(20);
    // 提示信息
    m_tipLabel = new DLabel(raiseWidget);
    m_tipLabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_tipLabel->setText(tr("Please do not forcibly shut down or cut power during the update, otherwise it will damage the system%1").arg("."));
    QFont font = m_tipLabel->font();
    font.setPixelSize(16);
    DFontSizeManager::instance()->unbind(m_tipLabel);
    m_tipLabel->setFont(font);
    QPalette palTip = m_tipLabel->palette();
    palTip.setColor(QPalette::WindowText, Qt::white);
    m_tipLabel->setPalette(palTip);
    mainLayout->addWidget(m_tipLabel, 0, Qt::AlignCenter);

    mainLayout->addSpacing(10);
    // 安装信息
    m_textEdit = new QTextEdit(raiseWidget);
    m_textEdit->setFixedSize(QSize(500, 500));
    m_textEdit->document()->setMaximumBlockCount(10);
    DFontSizeManager::instance()->bind(m_textEdit, DFontSizeManager::T6);
    QPalette pl = m_textEdit->palette();
    pl.setBrush(QPalette::Base, QBrush(QColor(255, 0, 0, 0)));
    m_textEdit->setPalette(pl);
    m_textEdit->setLineWidth(0);
    m_textEdit->setReadOnly(true);
    m_textEdit->setFont(font);
    m_textEdit->setTextColor(Qt::white);
    m_textEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    mainLayout->addWidget(m_textEdit, 0, Qt::AlignCenter);

    mainLayout->addStretch();
    lowerWidget->lower();
    raiseWidget->raise();
}

void OfflineUpgradeDialog::initData()
{
    m_progressBar->setValue(0);
    m_progressLabel->setText("0%");

    QTimer *timer = new QTimer(this);
    connect(timer, &QTimer::timeout, this, [=] {
        m_upgradeSec += 1;
        if (m_upgradeSec % 3 == 0) {
            m_tipLabel->setText(tr("Please do not forcibly shut down or cut power during the update, otherwise it will damage the system%1").arg("..."));
        } else if (m_upgradeSec % 3 == 1) {
            m_tipLabel->setText(tr("Please do not forcibly shut down or cut power during the update, otherwise it will damage the system%1").arg("."));
        } else if (m_upgradeSec % 3 == 2) {
            m_tipLabel->setText(tr("Please do not forcibly shut down or cut power during the update, otherwise it will damage the system%1").arg(".."));
        }
    });
    timer->start(1000);
}

// 设置进度
void OfflineUpgradeDialog::setUpgradeProgress(int progress)
{
    m_progressBar->setValue(progress);
    m_progressLabel->setText(QString("%1%").arg(progress));
}

// 设置安装信息
void OfflineUpgradeDialog::setUpgradeInfo(QString info)
{
    m_textEdit->append(info);
}

bool OfflineUpgradeDialog::event(QEvent *event)
{
    if (event->type() == QEvent::KeyPress
        || event->type() == QEvent::KeyRelease) {
        return false;
    }
    return QWidget::event(event);
}
